package cp213;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

// -------------------------------------------------------------------------------
/**
 * @author David Brown
 * @version 2017-11-15
 *
 *          Simple threaded counter.
 */
public class CountModel implements Runnable {
    // Initial counter value.
    private final static int MAX = 1000;
    // The current counter value.
    private int count = CountModel.MAX;
    // Allows views to listen to generic changes in the model.
    private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    // The suspend / resume flag.
    private boolean suspended = false;

    // -------------------------------------------------------------------------------
    /**
     * Attaches listeners to the model.
     *
     * @param listener
     */
    public void addPropertyChangeListener(
	    final PropertyChangeListener listener) {
	this.pcs.addPropertyChangeListener(listener);
    }

    // -------------------------------------------------------------------------------
    /**
     * @return Returns the current value of <code>count</code>.
     */
    public int getCount() {
	return this.count;
    }

    // -------------------------------------------------------------------------------
    /**
     * @return Returns the current value of <code>suspend</code>.
     */
    public boolean getSuspended() {
	return this.suspended;
    }

    // -------------------------------------------------------------------------------
    /*
     * (non-Javadoc)
     *
     * @see java.lang.Runnable#run()
     *
     * Must define block as {@code synchronized} in order to call {@code wait}.
     * Thread killed by external interruption by thread monitor owner.
     */
    @Override
    public synchronized void run() {

	try {
	    while (!Thread.interrupted()) {
		// Continue processing until the current thread is cancelled by
		// an interrupt.

		// Put the current thread to sleep for 500 ms.
		Thread.sleep(500);

		if (this.suspended) {
		    this.wait();
		} else {
		    this.count--;
		    // Inform listeners the model is updated.
		    this.pcs.firePropertyChange(null, null, null);
		}
	    }
	} catch (InterruptedException e) {
	    // Do Nothing.
	}
	return;
    }

    // -------------------------------------------------------------------------------
    /**
     * Sets the value of <code>suspend</code>
     *
     * @param suspended
     *            The new value for <code>suspend</code>
     */
    public void setSuspended(final boolean suspended) {
	this.suspended = suspended;
    }
}
