package cp213;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

// -------------------------------------------------------------------------------
/**
 * @author David B. Brown
 * @version 2017-11-15
 *
 *          A simple thread example.
 */
@SuppressWarnings("serial")
public class CountView extends JPanel {

    // -------------------------------------------------------------------------------
    /**
     * Displays the current value of the model counter.
     */
    private class DisplayListener implements PropertyChangeListener {

	@Override
	public void propertyChange(final PropertyChangeEvent arg0) {
	    countLabel.setText("" + model.getCount());
	}
    }

    // -------------------------------------------------------------------------------
    /**
     * Suspends/resumes the counter.
     */
    private class SuspendListener implements ActionListener {
	/**
	 * Suspends or resumes the model thread and updates the button text
	 * accordingly.
	 *
	 * @param evt
	 *            unused
	 */
	@Override
	public void actionPerformed(final ActionEvent evt) {

	    if (model.getSuspended()) {
		// Must define block as {@code synchronized} in order to call
		// {@code notify}.
		synchronized (model) {
		    model.setSuspended(false);
		    model.notify();
		}
		// Update the button text only after the model wakes up.
		suspendButton.setText("Suspend");
	    } else {
		// Must set model to 'wait' mode through the 'suspended' flag. A
		// direct call to {@code wait} from another object is not
		// allowed.
		model.setSuspended(true);
		suspendButton.setText("Resume");
	    }
	}
    }

    // -------------------------------------------------------------------------------
    // Displays current value of model counter.
    private final JLabel countLabel = new JLabel("", JLabel.CENTER);
    // The model to control.
    private final CountModel model;
    // Causes model to suspend or resume.
    private final JButton suspendButton = new JButton("Suspend");

    // -------------------------------------------------------------------------------
    /**
     * Constructor.
     *
     * @param model
     *            The model to display.
     */
    public CountView(final CountModel model) {
	this.model = model;
	this.layoutView();
	this.registerListeners();
    }

    // -------------------------------------------------------------------------------
    /**
     * Lays out the panel components.
     */
    private void layoutView() {
	this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	this.countLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
	this.countLabel.setText("" + this.model.getCount());
	this.add(this.countLabel);
	this.suspendButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	this.add(this.suspendButton);
    }

    // -------------------------------------------------------------------------------
    /**
     * Registers all listeners with the appropriate inner listener classes.
     */
    private void registerListeners() {
	this.suspendButton.addActionListener(new SuspendListener());
	this.model.addPropertyChangeListener(new DisplayListener());
    }

    // -------------------------------------------------------------------------------
}